@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -182,10 +182,11 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 				 uint8_t * data, size_t _data_size)
 {
 	uint16_t n_Y, n_g, n_p;
-	size_t _n_Y, _n_g, _n_p;
+	size_t _n_Y, _n_g, _n_p, _n_q;
 	uint8_t *data_p;
 	uint8_t *data_g;
 	uint8_t *data_Y;
+	uint8_t *data_q = NULL;
 	int i, bits, ret, p_bits;
 	unsigned j;
 	ssize_t data_size = _data_size;
@@ -245,6 +246,8 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 				session->internals.hsk_flags |= HSK_USED_FFDHE;
 				_gnutls_session_group_set(session, session->internals.priorities->groups.entry[j]);
 				session->key.proto.tls12.dh.params.qbits = *session->internals.priorities->groups.entry[j]->q_bits;
+				data_q = session->internals.priorities->groups.entry[j]->q->data;
+				_n_q = session->internals.priorities->groups.entry[j]->q->size;
 				break;
 			}
 		}
@@ -254,6 +257,14 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 		}
 	}
 
+#ifdef ENABLE_FIPS140
+	if (gnutls_fips140_mode_enabled() &&
+	    !_gnutls_dh_prime_match_fips_approved(data_p, n_p, data_g, n_g, NULL, NULL)) {
+		gnutls_assert();
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
+	}
+#endif
+
 	if (_gnutls_mpi_init_scan_nz(&session->key.proto.tls12.dh.params.params[DH_G], data_g, _n_g) != 0) {
 		gnutls_assert();
 		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
@@ -265,8 +276,19 @@ _gnutls_proc_dh_common_server_kx(gnutls_session_t session,
 		_gnutls_mpi_release(&session->key.proto.tls12.dh.params.params[DH_G]);
 		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
 	}
+	if (data_q && _gnutls_mpi_init_scan_nz(
+			    &session->key.proto.tls12.dh.params.params[DH_Q],
+			    data_q, _n_q) != 0) {
+		/* we release now because params_nr is not yet set */
+		_gnutls_mpi_release(
+			&session->key.proto.tls12.dh.params.params[DH_P]);
+		_gnutls_mpi_release(
+			&session->key.proto.tls12.dh.params.params[DH_G]);
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
+	}
 
-	session->key.proto.tls12.dh.params.params_nr = 3; /* include empty q */
+	/* include, possibly empty, q */
+	session->key.proto.tls12.dh.params.params_nr = 3;
 	session->key.proto.tls12.dh.params.algo = GNUTLS_PK_DH;
 
 	if (!(session->internals.hsk_flags & HSK_USED_FFDHE)) {
